(function ($) {
	'use strict';

	/**
	 * All of the code for your public-facing JavaScript source
	 * should reside in this file.
	 *
	 * Note: It has been assumed you will write jQuery code here, so the
	 * $ function reference has been prepared for usage within the scope
	 * of this function.
	 *
	 * This enables you to define handlers, for when the DOM is ready:
	 *
	 * $(function() {
	 *
	 * });
	 *
	 * When the window is loaded:
	 *
	 * $( window ).load(function() {
	 *
	 * });
	 *
	 * ...and/or other possibilities.
	 *
	 * Ideally, it is not considered best practise to attach more than a
	 * single DOM-ready or window-load handler for a particular page.
	 * Although scripts in the WordPress core, Plugins and Themes may be
	 * practising this, we should strive to set a better example in our own work.
	 */

	function prepareFormData(form, formData) {
		var list_ids = [];
		var is_multiple_lists = false;
		jQuery.each((form.serializeArray() || {}), function (i, field) {
			// Collect all list ids
			if(field.name === 'lists[]') {
				list_ids.push(field.value);
				is_multiple_lists = true;
			} else {
				formData['esfpx_' + field.name] = field.value;
			}
		});

		if(is_multiple_lists) {
			formData['esfpx_lists[]'] = list_ids;
		}


		return formData;
	}

	function handleResponse(response, form) {

		var status = response.status;

		var message_class = 'success';
		if(status === 'ERROR') {
			message_class = 'error';
		}

		var responseText = response['message_text'];
		var messageContainer = $(form).next('.es_subscription_message');
		messageContainer.attr('class', 'es_subscription_message ' + message_class);
		messageContainer.html(responseText);
	}

	$(document).ready(function () {

		// var submitButton = $('.es_subscription_form_submit');

		$(document).on('submit', '.es_subscription_form', function (e) {
			var form = $(this);
			var formData = {};
			formData = prepareFormData(form, formData);
			formData['es'] = 'subscribe';
			formData['action'] = 'es_add_subscriber';
			var actionUrl = es_data.es_ajax_url;
			jQuery(form).find('#spinner-image').show();
			$.ajax({
				type: 'POST',
				url: actionUrl,
				data: formData,
				dataType: 'json',
				success: function (response) {
					if( response && typeof response.status !== 'undefined' && response.status === "SUCCESS" ) {
						jQuery(form).slideUp('slow');
						jQuery(form).hide();
					} else {
						jQuery(form).find('#spinner-image').hide();
					}
					jQuery(window).trigger('es.send_response', [jQuery(form) , response]);
					handleResponse(response, form);
				},
				error: function (err) {
					jQuery(form).find('#spinner-image').hide();
					console.log(err, 'error');
				},
			});


			return false;
		});


	});
})(jQuery);
